#dragandrophandler {
    border: 2px solid #0B85A1;
    width: 100%;
    min-height: 200px;
    color: #92AAB0;
    text-align: center;
    vertical-align: middle;
    font-size: 200%;
}

/*Node*/

.node{
    -moz-border-radius: 4px;
    border: 1px solid #ddd;
    padding: 3px;
    margin-bottom: 5px;
}

.chosen{
    background-color: rgba(0, 255, 0, 0.4);
}

.chosen, .not-chosen{
    transition: background-color 1s;
}

.hostname, .ip{
    font-weight: bold;
}

.alert.alert-success{
    display:none;
}

.output{
    overflow-y: scroll;
    height: 200px;
    max-height: 200px;
}


/*Console outputs*/
.stdout, .stderr{
    font-family: "Courier New", Courier, monospace;
}

.stdout{
    color: #000;
}

.stderr{
    color: #f00;
}

.logger-badge{
    background: #f00;
}

.panel-heading p a{
    color:#fff;
    text-decoration: underline;
}

#command{
    float: none;
    margin: 0 auto;
}

#command input{
    font-family: "Lucida Console", Monaco, monospace;
}

.memorylegend{
    text-align: center;
    color: #fff;
}

.free{
    background-color: #98ABC5;
}
.used{
    background-color: #8A89A6;
}

.cached{
    background-color: #7B6888;
}

.top-percentage{
    color: rgba(14, 14, 14, 0.6);
}